// C++ std libs
#include <iostream>
#include <vector>
#include <map>
#include <string>
#include <fstream>
#include <sstream>
#include <stdexcept>
#include <sys/time.h>
#include <sys/types.h>

// C++ Boost
#include <boost/program_options.hpp>

// utils
#include "utils.h"

// Classes
#include "Vihgo.h"

using namespace std;

Vihgo::Vihgo()
{
	//
}
Vihgo::Vihgo( string iBF , string gF, string intF, string protF, string refF, string outF, string cResF, string csF, string iRCB, int dT, int nT, int lL, double rT ,double pvT, bool countStop )
{
	this->inputBamFile = iBF;
	this->geneFile = gF;
	this->integraseFile = intF;
	this->proteaseFile = protF;
	this->referenceFile = refF;
	this->outputFile = outF;
	this->codonResultFile = cResF;
	this->csFile = csF;
	this->inRunControlBam = iRCB;
	this->depthThreshold = dT;
	this->nbThread = nT;
	this->loggingLevel = lL;
	this->ratioThreshold = rT;
	this->pvalueThreshold = pvT;
	this->countStop = countStop;

}

int Vihgo::mainLoop()
{
	this->getGenome();
	//this->displayGenome();
	this->getGenes();
	this->displayGenes();

	return 0;
}

int Vihgo::getGenome( )
{
	struct timeval tbegin, tend;
	string ligne;
	char mot;
	string header = ">";
	int i = 0;
	string currentChr;

	// fasta parsing
	ifstream fastaStream ( this->referenceFile.c_str() );
	if ( fastaStream )
	{
		while ( getline ( fastaStream, ligne) )
		{
			if( ligne.find( header ) == 0 )
			{
				ligne.replace(0,1,"");
				currentChr = ligne;
				map< int , char> tmpMap;
				this->referenceSequence[currentChr] = tmpMap;
			}
			else
			{
				for( unsigned int k = 0 ; k < ligne.length() ; k ++ )
				{
					mot = ligne[k];
					mot = checkBase( mot );

					this->referenceSequence[currentChr][i] = mot;
					i ++;
				}
			}
		}
	}
	return 0;
}

void Vihgo::displayGenome()
{
	map<string, map<int, char>>::iterator myIterA ;

	for( myIterA = this->referenceSequence.begin() ; myIterA != this->referenceSequence.end() ; myIterA ++ )
	{
		cerr << "Chr " << myIterA->first << " : " << endl;
		map<int, char> tmpMap;
		tmpMap  = myIterA->second;
		map<int, char>::iterator myIterB;
		for( myIterB = tmpMap.begin() ; myIterB != tmpMap.end() ; myIterB ++ )
		{
			cerr << myIterB->first << " : " << myIterB->second << " ; ";
		}
		cerr << endl;
	}

}

int Vihgo::getGenes()
{
	struct timeval tbegin, tend;
	string ligne;
	string mot;
	string header = ">";
	int i = 0;
	string currentChr;
	string chromosome;
	string cds;
	string annot;
	string start;
	string end;
	string strand;


	// fasta parsing
	ifstream geneStream ( this->geneFile.c_str() );
	if ( geneStream )
	{
		while ( getline ( geneStream, ligne) )
		{
			i = 0;
			// get information from the line
			istringstream issLigne (ligne);
			while ( getline( issLigne, mot, '\t' ) )
			{
				switch( i )
				{
					case 0 :
						chromosome = mot;
						break;
					case 2 :
						cds = mot;
						break;
					case 3 :
						start= mot;
						break;
					case 4 :
						end = mot;
						break;
					case 6 :
						strand = mot;
						break;
					case 8 :
						annot = mot;
						break;
					default :
						break;
				}
				i ++;
			}

			if( cds == "CDS" )
			{
				istringstream issAnnot (annot);
				while ( getline( issAnnot, mot, ';' ) )
				{
					if( mot.find( "protein_id" ) == 0 )
					{
						istringstream issMot( mot );
						string subMot;
						string g;
						i = 0;
						while ( getline( issMot, subMot, '=' ) )
						{
							if( i == 1 )
							{
								g = subMot;
							}
							i ++;
						}

						// modify gene string : deleting special chars
						// cerr << "Gene before : " << subMot << endl;
						subMot = pyReplace(subMot , "\"" , "");
						// cerr << "Gene after : " << subMot << endl;

						// test if gene is already known
						if (this->genes.count( subMot ) == 0 )
						{
							cerr << "Creating new gene : " << subMot << endl;
							map<string , string> tmpMap;
							map<int, string> tmpMapT;
							this->genes[subMot] = tmpMap;
							this->referenceCodon[subMot] = tmpMapT;
							this->genes[subMot]["start"] = "";
							this->genes[subMot]["end"] = "";
							this->genes[subMot]["strand"] = "";
							this->genes[subMot]["chrom"] = chromosome;
						}

						// add informations
						if( (this->genes[subMot]["start"]).length() > 0 )
						{
							(this->genes[subMot]["start"]).append( "," );
						}
						(this->genes[subMot]["start"]).append( start );

						if( (this->genes[subMot]["end"]).length() > 0 )
						{
							(this->genes[subMot]["end"]).append( ",");
						}
						(this->genes[subMot]["end"]).append( end );

						if( (this->genes[subMot]["strand"]).length() > 0 )
						{
							(this->genes[subMot]["strand"]).append(",");
						}
						(this->genes[subMot]["strand"]).append( strand );
					}
				}
			}
		}
	}
	return 0;

}

void Vihgo::displayGenes()
{
	cerr << "Displaying gene list : " << endl;
	map<string, map< string , string>>::iterator myIterB;
	for( myIterB = this->genes.begin() ;myIterB != this->genes.end() ; myIterB ++ )
	{
		cerr << "Gene : " << myIterB->first << " : " << endl;
		map< string , string> tmpMap;
		tmpMap = myIterB->second;

		cerr << "chr : " << tmpMap["chrom"] << " ; start : " << tmpMap["start"] << " ; end : " << tmpMap["end"] << " ; strand : " << tmpMap["strand"] << endl;
	}
}















// getters



// setters
